"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyMutation = exports.deleteValue = void 0;
exports.deleteValue = Symbol('deleteValue');
function applyMutation(conditions, obj) {
    // Create the `deleteValue` symbol
    var _a;
    // Start with a shallow copy of the original object
    let result = Object.assign({}, obj);
    // Recursively apply the mutation to all properties of the object, regardless of
    // whether or not they have a `mutate` property
    for (const [key, value] of Object.entries(obj)) {
        if (value && typeof value == 'object') {
            result[key] = applyMutation(conditions, value);
        }
    }
    // Iterate over the list of conditions and apply the corresponding mutation
    // to the result object
    for (const condition of conditions) {
        let conditionKey;
        let args;
        if (typeof condition != 'string') {
            conditionKey = condition.condition;
            args = condition.args();
        }
        else
            conditionKey = condition;
        const mutation = (_a = obj.mutate) === null || _a === void 0 ? void 0 : _a[conditionKey];
        if (mutation) {
            for (const [key, value] of Object.entries(mutation)) {
                if (value === exports.deleteValue) {
                    delete result[key];
                }
                else if (typeof value == 'function') {
                    result[key] = value.bind(this)(result[key], ...(Array.isArray(args) ? args : [args]));
                }
                else {
                    result[key] = value;
                }
            }
        }
    }
    // Remove any remaining `mutate` properties from the result
    for (const [key, value] of Object.entries(result)) {
        if (key === 'mutate') {
            delete result[key];
        }
    }
    // Return the result
    return result;
}
exports.applyMutation = applyMutation;
